#include "repeater.h"

MODULE_LICENSE("Dual BSD/GPL");

static dev_t dev_n;
static struct repeater_dev *repeaters;

static int devices = 1;
module_param(devices, int, S_IRUGO);

struct file_operations repeater_fops = {
	.owner   = THIS_MODULE,
	.open    = repeater_open,
	.write   = repeater_write,
	.read    = repeater_read,
	.release = repeater_release,
};


static int repeater_init(void){
	int result = alloc_chrdev_region(&dev_n,0,devices,"repeater");
	if(!result){
		int repeater_major = MAJOR(dev_n);
		result = -ENOMEM;
		repeaters = kmalloc(devices * sizeof(struct repeater_dev),GFP_KERNEL);
		if(repeaters){
			int i,err;
			for(i=0;i<devices;i++){
				int devno=MKDEV(repeater_major,i);
				err=0;
				repeaters[i].size=0;
				sema_init(&(repeaters[i].sem), 1);
				cdev_init(&(repeaters[i].cdev), &repeater_fops);
				repeaters[i].cdev.owner = THIS_MODULE;
				repeaters[i].cdev.ops = &repeater_fops;
				err = cdev_add(&(repeaters[i].cdev), devno, 1);
				if(err)
					printk(KERN_NOTICE "Error %d adding repeater%d", err, i);
			}
			return 0;
		}
		unregister_chrdev_region(dev_n,devices);
	}
	return result;
}

static void repeater_exit(void){
	kfree(repeaters);
	unregister_chrdev_region(dev_n,devices);
}

module_init(repeater_init);
module_exit(repeater_exit);

