#include <linux/init.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/slab.h>

#include <asm/uaccess.h>

MODULE_LICENSE("Dual BSD/GPL");

ssize_t to_dmesg_write(struct file *filp, const char __user *buff, size_t count, loff_t *offp);

struct file_operations to_dmesg_fops = {
	.owner = THIS_MODULE,
	.write = to_dmesg_write,
};

static dev_t dev_n;
static struct cdev *my_cdev;

static int to_dmesg_init(void){
	int result = alloc_chrdev_region(&dev_n,0,1,"to-dmesg");
	if(!result){
		my_cdev = cdev_alloc();
		my_cdev -> owner = THIS_MODULE;
		my_cdev -> ops = &to_dmesg_fops;
		result = cdev_add(my_cdev, dev_n, 1);
		if(!result){
			return 0;
		}
		unregister_chrdev_region(dev_n,1);
	}
	return result;
}

static void to_dmesg_exit(void){
	cdev_del(my_cdev);
	unregister_chrdev_region(dev_n,1);
}

ssize_t to_dmesg_write(struct file *filp, const char __user *buff, size_t count, loff_t *offp){
	int retval = -ENOMEM;
	char *my_buff=(char *)kmalloc(count+1,GFP_KERNEL);
	if(my_buff){
		retval = -EFAULT;
		if(!copy_from_user(my_buff,buff,count)){
			retval = count;
			my_buff[count]='\0';
			printk(KERN_ALERT "%s",my_buff);
		}
		kfree(my_buff);
	}
	return retval;
}

module_init(to_dmesg_init);
module_exit(to_dmesg_exit);

